/**
 * A simple GSM alarm based on off-the-shelf Siemens TC65
 * GSM/GPRS terminal.
 * 
 *  GSM-   GSM/GPRS  Siemens TC65.
 *      .
 * 
 *      
 * 
 * : GPIO0, GPIO1 - 
 * : GPIO5, GPIO6
 * :  ,   RS-232 (115200, 8, n, 1)
 * (, HyperTerminal, term95  minicom).
 * 
 * 
 * Connect inputs to GPIO0, GPIO1
 * 
 * Author: "Dmitry" <dkorobkov@mail.ru>
 */

package readUSSD;

import javax.microedition.midlet.*;

import com.siemens.icm.io.*;

public class READUssd extends MIDlet{
	ATCommand atCommand;
	MyATCommandListener atCommandListener; //  -    . 
	Tc65SerialPort	tc65SerialPort;
	int Animation=0;
	String strUSSD;
	
	public READUssd()
	{
	    try 
	    {
	        atCommand = new ATCommand(false);
	        tc65SerialPort = new Tc65SerialPort();
	        atCommandListener = new MyATCommandListener(false, tc65SerialPort);
	        strUSSD = new String();
	        
	        //   atCommandListener    -,
	        //     ( RING)
	        atCommand.addListener(atCommandListener);
	        
	        serialOut("\n\r\n\rUSSD receiver Java app started (090814).\n\r");
	    } 
	    catch(ATCommandFailedException e) 
	    {
	        serialOut(e.getMessage());
	        notifyDestroyed();
	    }
	    catch(Exception e) 
	    {
	        System.out.println(e);
	        notifyDestroyed();
	    }
	}

	/**
	 * Wrappers for debug output to serial port
	 * @param str
	 */
	void serialOut(String str)
	{
		tc65SerialPort.serialOut(str.getBytes());
	}
	void serialOut(byte[] ar)
	{
		tc65SerialPort.serialOut(ar);
	}
	
	/**
	   * startApp()
	   */
	
	public void startApp() throws MIDletStateChangeException 
	{
		serialOut("USSD receiver (build 0.1) is ready for operation.\n\r" +
				"Enter USSD starting with * and ending with #, press Q to quit.\n\r\n\r");

		boolean bEnd = false;
		while(bEnd == false)
		{
			try {
				Thread.sleep(250);
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			int ch;
			char c;
			if(tc65SerialPort.hasData() > 0)
			{
			    ch = tc65SerialPort.readByte();
			    c = (char) ch;
			    switch(ch) 
			    {
			    case '#':
			    	serialOut("Sending ATD" + strUSSD + "#;...\r\n");
			    	try {
			    		serialOut("atCommand.send() returns " + 
			    				atCommand.send("ATD" + strUSSD + "#;\r" + "\r\n"));
					} catch (IllegalStateException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IllegalArgumentException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (ATCommandFailedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
			    break;
			    case '*':
			    	strUSSD = "*";
			    	break;
			    case '1': case '2': case '3': case '4': case '5': 
			    case '6': case '7': case '8': case '9': case '0':
			    	strUSSD += c;
			    	break;
			    case 'Q':
			    case 'q': 
			    	bEnd = true;
			    	break;
			    case 0x0d: case  0x0a: break;
			    default:
			    {
			    	serialOut("Unknown option. Press R to send \"*100#\" request, Q to quit.\n\r");
			    }
			    break;
			
			    }
			}
			ProcessUnsolicitedEvents();
			Animate();
		}
		serialOut("\r\nGoodbye.\r\n");
		destroyApp(true);
	}
		
	void Animate()
	{
		switch(Animation)
		{
		case 0:
			serialOut("\b|");
			break;
		case 1:
			serialOut("\b/");
			break;
		case 2:
			serialOut("\b-");
			break;
		case 3:
			serialOut("\b\\");
			break;
		}
		Animation++; 
		if(Animation > 3) Animation = 0;
	}
		

/**
 *   ,    ( RING  
 *    SMS)
*/
	void ProcessUnsolicitedEvents()
	{
  	  if(atCommandListener.hasNewEvent())
	  {
  		serialOut("\n\rNew event:" + atCommandListener.strMessage + "\n\r");
 	  }
	}
	
	  /**
	   * pauseApp()
	   */
	  public void pauseApp() {
//	    System.out.println("RS232Demo: pauseApp()");
	  }

	  /**
	   * destroyApp()
	   *
	   * This is important.  It closes the app's RecordStore
	   * @param cond true if this is an unconditional destroy
	   *             false if it is not
	   *             currently ignored and treated as true
	   */
	  public void destroyApp(boolean cond) {
//		  System.out.println("RS232Demo: destroyApp(" + cond + ")");
		  //  ,   

		  notifyDestroyed();
//		  tc65SerialPort.serialOut("destroyApp");
	  }
	  
}
